"""\
This file is an installation script for PlonePhotoAlbum.  It's meant
to be used as an External Method.  To use, add an external method to the
root of the CMF Site that you want ZPT skins registered in with the
configuration:

 id: install_photoalbum
 title: Install Photo Album Product *optional*
 module name: PlonePhotoAlbum.Install
 function name: install

Then go to the management screen for the newly added external method
and click the 'Test' tab.  The install function will execute and give
information about the steps it took to register and install the
ZPT skins into the CMF Site instance. 
"""


from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName
from Products.ExternalMethod import ExternalMethod

from Acquisition import aq_base
from cStringIO import StringIO
import string

from Products.PlonePhotoAlbum.config import PROJECTNAME, GLOBALS, PORTAL_TYPE, SKINS_DIR
from Products.PlonePhotoAlbum.PhotoAlbum import factory_type_information

def install(self):
    " Register the with portal_types and friends "
    out = StringIO()
    typestool = getToolByName(self, 'portal_types')
    skinstool = getToolByName(self, 'portal_skins')
    portal_url = getToolByName(self, 'portal_url')

    # Borrowed from CMFDefault.Portal.PortalGenerator.setupTypes()
    # We loop through anything defined in the factory type information
    # and configure it in the types tool if it doesn't already exist
    for t in factory_type_information:
        if t['id'] not in typestool.objectIds():
            cfm = apply(ContentFactoryMetadata, (), t)
            typestool._setObject(t['id'], cfm)
            out.write('Registered with the types tool\n')
        else:
            out.write('Object "%s" already existed in the types tool\n' % (
                t['id']))

    out.write('Registered with portal_types\n')

    # Setup the skins
    # This is borrowed from CMFDefault/scripts/addImagesToSkinPaths.pys
    if SKINS_DIR not in skinstool.objectIds():
        # We need to add Filesystem Directory Views for any directories
        # in our skins/ directory.  These directories should already be
        # configured.
        addDirectoryViews(skinstool, 'skins', GLOBALS)
        out.write("Added %s directory view to portal_skins\n"%SKINS_DIR)


    # Now we need to go through the skin configurations and insert
    # our skin into the configurations.  Preferably, this should be
    # right after where 'custom' is placed.  Otherwise, we append
    # it to the end.
    skins = skinstool.getSkinSelections()
    for skin in skins:
        path = skinstool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        if SKINS_DIR not in path:
            try: path.insert(path.index('custom')+1, SKINS_DIR)
            except ValueError:
                path.append(SKINS_DIR)
            
            path = string.join(path, ', ')
            # addSkinSelection will replace exissting skins as well.
            skinstool.addSkinSelection(skin, path)
            out.write("Added %s to %s skin\n" % (SKINS_DIR,skin))
        else:
            out.write("Skipping %s skin, %s is already set up\n" % (
                skin, SKINS_DIR))

    ##
    ## Register PhotoAlbum as folderish content in plone, so it will
    ## get the appropriate folder_contents tabs
    try:
        props = self.portal_properties.site_properties
        tabs = getattr(props, 'use_folder_tabs')
        contents = getattr(props, 'use_folder_contents')
    
        if PORTAL_TYPE not in tabs:
            newtabs = list(tabs)
            newtabs.append(PORTAL_TYPE)
            props._updateProperty('use_folder_tabs', newtabs)
    
        if PORTAL_TYPE not in contents:
            newcontents = list(contents)
            newcontents.append(PORTAL_TYPE)
            props._updateProperty('use_folder_contents', newcontents)
            
        out.write("Added %s to folder tabs (plone)\n" % PORTAL_TYPE)
    except:
        out.write("not a plone site, no folder tabs property\n")
        


    out.write("Successfully installed %s." % PROJECTNAME)
    return out.getvalue()
